@@ -4,10 +4,16 @@ mod buffer;
 mod bytes;
 mod dict;
 mod fragment;
+#[cfg(Py_3_12)]
+mod immortal;
 mod list;
+mod long;
 
 pub use buffer::*;
 pub use bytes::*;
 pub use dict::*;
 pub use fragment::{orjson_fragmenttype_new, Fragment};
+#[cfg(Py_3_12)]
+pub use immortal::_Py_IsImmortal;
 pub use list::PyListIter;
+pub use long::{pylong_is_unsigned, pylong_is_zero};
