@@ -44,7 +44,7 @@ pub fn serialize(
             Ok(buf.finish())
         }
         Err(err) => {
-            ffi!(_Py_Dealloc(buf.finish().as_ptr()));
+            ffi!(_Py_Dealloc(buf.bytes_ptr().as_ptr()));
             Err(err.to_string())
         }
     }
